@@ -41,10 +41,7 @@ import inspect
 import logging
 import importlib
 import cmdtools
-
-
-class DuplicateCommandNameError(Exception):
-    pass
+from typing import Any, Callable, List, Union, Optional
 
 
 class RunnerError(Exception):
@@ -54,120 +51,10 @@ class RunnerError(Exception):
     """
 
 
-class CommandWrapperObject:
-    def __init__(self):
-        self.name = None
-        self.aliases = []
-        self.callback = None
-        self.error_callback = None
-
-    def error(self, func):
-        self.error_callback = func
-
-        def wrapper(*args, **kwargs):
-            return func(*args, **kwargs)
-        return wrapper
-
-    def __call__(self):
-        return self.callback(*self._args, **self._kwargs)
-
-    def _checkfunc(self, func) -> bool:
-        """check is object callable"""
-        if func is not None and (
-            inspect.isfunction(func)
-            or inspect.ismethod(func)
-            or inspect.iscoroutinefunction(func)
-        ):
-            return True
-
-        return False
-
-    def is_coroutine(self) -> bool:
-        """check whether command is coroutine or not, based by the callbacks"""
-        if self.has_callback():
-            return inspect.iscoroutinefunction(self.callback)
-        elif self.has_callback() and self.has_error_callback():
-            return inspect.iscoroutinefunction(
-                self.callback
-            ) and inspect.iscoroutinefunction(self.error_callback)
-
-        # not a coroutine or object does not have command callback
-        return False
-
-    def has_callback(self) -> bool:
-        """check whether callback is exist or not"""
-        return self._checkfunc(self.callback)
-
-    def has_error_callback(self) -> bool:
-        """check whether error callback is exist or not"""
-        return self._checkfunc(self.error_callback)
-
-
-class CommandWrapper:
-    """wrapper for commands (?)"""
-    def __init__(self, commands: list = None):
-        if commands is None:
-            commands = []
-
-        self.commands = commands
-
-    def command(self, *args, **kwargs):
-        """base decorator for callbacks"""
-        def decorator(obj):
-            if (inspect.isfunction(obj), inspect.ismethod(obj), inspect.iscoroutinefunction(obj)).count(True) > 0:
-                wrapper = CommandWrapperObject()
-
-                for attr in kwargs:
-                    setattr(wrapper, attr, kwargs[attr])
-
-                wrapper.name = kwargs.get('name', obj.__name__)
-                wrapper.aliases = kwargs.get('aliases', [])
-                wrapper.callback = obj
-                self.commands.append(wrapper)
-
-                def func_wrapper(*args, **kwargs):
-                    setattr(wrapper, "_args", args)
-                    setattr(wrapper, "_kwargs", kwargs)
-                    return wrapper
-                return func_wrapper()
-            elif inspect.isclass(obj):
-                if isinstance(obj(), CommandObject):
-                    self.commands.append(obj())
-            else:
-                raise TypeError("Cannot assign command for object:", type(obj))
-        return decorator
-
-    async def run(self, cmd, attrs: dict = None):
-        if attrs is None:
-            attrs = {}
-
-        if not isinstance(cmd, (cmdtools.Cmd, cmdtools.AioCmd)):
-            raise TypeError(f"object {type(cmd)} is not a Cmd or AioCmd instance")
-
-        for command in self.commands:
-            if command.name == cmd.name or cmd.name in command.aliases:
-                args = []
-                if command.has_callback():
-                    args.append(command.callback)
-                else:
-                    return logging.error(
-                        f"Command name '{command.name}' has no callback"
-                    )
-                if command.has_error_callback():
-                    args.append(command.error_callback)
-
-                if command.is_coroutine():
-                    return await cmd.process_cmd(*args, attrs=attrs)
-
-                return cmd.process_cmd(*args, attrs=attrs)
-
-        raise RunnerError(f"Couln't find command '{cmd.name}'")
-
-
 class CommandObject:
     """command object container"""
 
-    def __init__(self, object_, name: str = None):
+    def __init__(self, object_: Union[Callable, object], name: str = None):
         self.object = object_
 
         # try get the command name from the module or class first
@@ -175,18 +62,22 @@ class CommandObject:
         self.name = getattr(self.object, "name", name)  # type: str
 
     @property
-    def aliases(self) -> list:
-        return getattr(self.object, "_aliases", [])
+    def aliases(self) -> List[str]:
+        """get command aliases"""
+        return getattr(self.object, "__aliases__", [])
 
     @property
-    def callback(self):
+    def callback(self) -> Optional[Callable]:
+        """get command callback"""
         return getattr(self.object, self.name, None)
 
     @property
-    def error_callback(self):
+    def error_callback(self) -> Optional[Callable]:
+        """get command error callback"""
         return getattr(self.object, "error_" + self.name, None)
 
-    def _checkfunc(self, func) -> bool:
+    @classmethod
+    def _checkfunc(cls, func) -> bool:
         """check is object callable"""
         if func is not None and (
             inspect.isfunction(func)
@@ -201,7 +92,7 @@ class CommandObject:
         """check whether command is coroutine or not, based by the callbacks"""
         if self.has_callback():
             return inspect.iscoroutinefunction(self.callback)
-        elif self.has_callback() and self.has_error_callback():
+        if self.has_callback() and self.has_error_callback():
             return inspect.iscoroutinefunction(
                 self.callback
             ) and inspect.iscoroutinefunction(self.error_callback)
@@ -221,17 +112,131 @@ class CommandObject:
 class Command(CommandObject):
     """main command class inheritance"""
 
-    def __init__(self, name):
+    def __init__(self, name: str):
         super().__init__(name=name, object_=self)
 
 
+class CommandWrapperObject:
+    """instance of wrapped command object"""
+
+    def __init__(self):
+        self.name = None
+        self.aliases = []
+        self.callback = None
+        self.error_callback = None
+
+    def error(self, func: Callable) -> Callable:
+        """error handler function wrapper"""
+        self.error_callback = func
+
+        def wrapper(*args, **kwargs):
+            return func(*args, **kwargs)
+        return wrapper
+
+    def __call__(self):
+        return self.callback(*getattr(self, '_args', ()), **getattr(self, '_kwargs', {}))
+
+    @classmethod
+    def _checkfunc(cls, func: Callable) -> bool:
+        """check is object callable"""
+        if func is not None and (
+            inspect.isfunction(func)
+            or inspect.ismethod(func)
+            or inspect.iscoroutinefunction(func)
+        ):
+            return True
+
+        return False
+
+    def is_coroutine(self) -> bool:
+        """check whether command is coroutine or not, based by the callbacks"""
+        if self.has_callback():
+            return inspect.iscoroutinefunction(self.callback)
+        if self.has_callback() and self.has_error_callback():
+            return inspect.iscoroutinefunction(
+                self.callback
+            ) and inspect.iscoroutinefunction(self.error_callback)
+
+        # not a coroutine or object does not have command callback
+        return False
+
+    def has_callback(self) -> bool:
+        """check whether callback is exist or not"""
+        return self._checkfunc(self.callback)
+
+    def has_error_callback(self) -> bool:
+        """check whether error callback is exist or not"""
+        return self._checkfunc(self.error_callback)
+
+
+class CommandWrapper:
+    """wrapper for commands (?)"""
+    def __init__(self, commands: List[CommandObject] = None):
+        if commands is None:
+            commands = []
+
+        self.commands = commands
+
+    def command(self, **kwargs) -> Optional[CommandWrapperObject]:
+        """base decorator for callbacks"""
+        def decorator(obj):
+            if (inspect.isfunction(obj), inspect.ismethod(obj), inspect.iscoroutinefunction(obj)).count(True) > 0:
+                wrapper = CommandWrapperObject()
+
+                for attr in kwargs.items():
+                    setattr(wrapper, attr[0], attr[1])
+
+                wrapper.name = kwargs.get('name', obj.__name__)
+                wrapper.aliases = kwargs.get('aliases', [])
+                wrapper.callback = obj
+                self.commands.append(wrapper)
+
+                def func_wrapper(*args, **kwargs):
+                    setattr(wrapper, "_args", args)
+                    setattr(wrapper, "_kwargs", kwargs)
+                    return wrapper
+                return func_wrapper()
+
+            if inspect.isclass(obj):
+                if CommandObject in inspect.getmro(obj):
+                    self.commands.append(obj())
+            else:
+                raise TypeError("Cannot assign command for object:", type(obj))
+        return decorator
+
+    async def run(self, cmd: Union[cmdtools.Cmd, cmdtools.AioCmd], attrs: dict = None) -> Any:
+        """run command instance"""
+        if attrs is None:
+            attrs = {}
+
+        if not isinstance(cmd, (cmdtools.Cmd, cmdtools.AioCmd)):
+            raise TypeError(f"object {type(cmd)} is not a Cmd or AioCmd instance")
+
+        for command in self.commands:
+            if command.name == cmd.name or cmd.name in command.aliases:
+                args = []
+                if command.has_callback():
+                    args.append(command.callback)
+                else:
+                    return logging.error("Command name '%s' has no callback", command.name)
+                if command.has_error_callback():
+                    args.append(command.error_callback)
+
+                if command.is_coroutine():
+                    return await cmd.process_cmd(*args, attrs=attrs)
+
+                return cmd.process_cmd(*args, attrs=attrs)
+
+        raise RunnerError(f"Couln't find command '{cmd.name}'")
+
+
 class CommandRunner:
     """command single runner class"""
 
     def __init__(self, command: CommandObject):
         self.command = command
 
-    async def run(self, cmd: cmdtools.Cmd, attrs: dict = None):
+    async def run(self, cmd: Union[cmdtools.Cmd, cmdtools.AioCmd], attrs: dict = None) -> Any:
         """run command from parsed command object"""
 
         if not isinstance(cmd, (cmdtools.Cmd, cmdtools.AioCmd)):
@@ -245,9 +250,7 @@ class CommandRunner:
             if self.command.has_callback():
                 args.append(self.command.callback)
             else:
-                return logging.error(
-                    f"Command name '{self.command.name}' has no callback"
-                )
+                return logging.error("Command name '%s' has no callback", self.command.name)
             if self.command.has_error_callback():
                 args.append(self.command.error_callback)
 
@@ -262,10 +265,10 @@ class CommandRunner:
 class CommandRunnerContainer:
     """command runner container class inheritance"""
 
-    def __init__(self, commands: list):
+    def __init__(self, commands: List[CommandObject]):
         self.commands = commands
 
-    async def run(self, cmd: cmdtools.Cmd, attrs: dict = None):
+    async def run(self, cmd: Union[cmdtools.Cmd, cmdtools.AioCmd], attrs: dict = None) -> Any:
         """run command from parsed command object"""
 
         if not isinstance(cmd, (cmdtools.Cmd, cmdtools.AioCmd)):
@@ -280,9 +283,7 @@ class CommandRunnerContainer:
                 if command.has_callback():
                     args.append(command.callback)
                 else:
-                    return logging.error(
-                        f"Command name '{command.name}' has no callback"
-                    )
+                    return logging.error("Command name '%s' has no callback", command.name)
                 if command.has_error_callback():
                     args.append(command.error_callback)
 
@@ -297,16 +298,16 @@ class CommandRunnerContainer:
 class CommandModule(CommandRunnerContainer):
     """command module container class"""
 
-    def __init__(self, filename: str, **attrs):
+    def __init__(self, filename: str, *, load_classes: bool = True):
         self.filename = filename
         self.commands = []
 
-        self.load_classes = attrs.get("load_classes", True)
+        self.load_classes = load_classes
 
         self.load_module(load_classes=self.load_classes)
         super().__init__(commands=self.commands)
 
-    def load_module(self, load_classes):
+    def load_module(self, load_classes: bool) -> None:
         """load command classes from a module"""
         if self.filename.endswith(".py"):
             modulestr = (
@@ -319,7 +320,7 @@ class CommandModule(CommandRunnerContainer):
                     obj_ = getattr(module, obj, None)
 
                     if inspect.isclass(obj_) and obj_.__module__ == module.__name__:
-                        if isinstance(obj_(), CommandObject):
+                        if CommandObject in inspect.getmro(obj_):
                             self.commands.append(obj_())
             else:
                 self.commands.append(CommandObject(module))
@@ -328,16 +329,16 @@ class CommandModule(CommandRunnerContainer):
 class CommandDir(CommandRunnerContainer):
     """command directory container class"""
 
-    def __init__(self, rootdir: str, **attrs):
+    def __init__(self, rootdir: str, *, search_tree: bool = False, load_classes: bool = False):
         self.rootdir = rootdir.replace("/", os.sep).replace("\\", os.sep)
         self.commands = []
-        self.search_tree: bool = attrs.get("search_tree", False)
-        self.load_classes: bool = attrs.get("load_classes", False)
+        self.search_tree: bool = search_tree
+        self.load_classes: bool = load_classes
 
         self.load_commands(search_tree=self.search_tree, load_classes=self.load_classes)
         super().__init__(commands=self.commands)
 
-    def load_commands(self, search_tree, load_classes):
+    def load_commands(self, search_tree: bool, load_classes: bool) -> None:
         """load commands from files inside rootdir"""
         dirs = []
         if search_tree:
@@ -355,7 +356,7 @@ class CommandDir(CommandRunnerContainer):
                         + "."
                         + file.rsplit(".py", 1)[0]
                     )
-                    if self.load_classes is False:
+                    if load_classes is False:
                         self.commands.append(
                             CommandObject(module, file.rsplit(".py", 1)[0])
                         )
@@ -364,11 +365,11 @@ class CommandDir(CommandRunnerContainer):
                             obj_ = getattr(module, obj, None)
 
                             if inspect.isclass(obj_) and obj_.__module__ == module.__name__:
-                                if isinstance(obj_(), CommandObject):
+                                if CommandObject in inspect.getmro(obj_):
                                     self.commands.append(obj_())
 
 
-def get_command_names(commands: list, get_aliases=False):
+def get_command_names(commands: List[CommandObject], get_aliases=False) -> List[str]:
     """get command names from command list of CommandObject"""
     names = []
     for command in commands:
